@@ -604,9 +604,7 @@ macro_rules! parse_body {
         let mut reader = $reader::new($input);
         let mut buf = String::new();
 
-        loop {
-            read_line!(reader, &mut buf)?;
-
+        while read_line!(reader, &mut buf)? != 0 {
             // Check for character U+FEFF specifically thanks to map id 797130
             if !buf
                 .trim_matches(|c: char| c.is_whitespace() || c == '﻿')
@@ -855,9 +853,10 @@ mod tests {
     #[cfg(feature = "async_tokio")]
     #[test]
     fn parsing_async_tokio() {
-        use tokio::{fs::File, runtime::Runtime};
+        use tokio::{fs::File, runtime::Builder};
 
-        Runtime::new()
+        Builder::new_current_thread()
+            .build()
             .expect("could not start runtime")
             .block_on(async {
                 let map_id = map_id();
